/*******************************************************************
 *
 *  ttconfig.h                                                1.0   
 *
 *    Configuration settings header file (spec only).            
 *
 *  Copyright 1996-1998 by
 *  David Turner, Robert Wilhelm, and Werner Lemberg.
 *
 *  This file is part of the FreeType project, and may only be used
 *  modified and distributed under the terms of the FreeType project
 *  license, LICENSE.TXT.  By continuing to use, modify, or distribute 
 *  this file you indicate that you have read the license and
 *  understand and accept it fully.
 *
 *  Notes:
 *
 *    All the configuration #define statements have been gathered in
 *    this file to allow easy check and modification.       
 *
 ******************************************************************/

#ifndef TTCONFIG_H
#define TTCONFIG_H


/* Here we include the file ft_conf.h for system dependent stuff.        */
/* The specific makefile is responsible for providing the right path to  */
/* this file.                                                            */

#include "ft_conf.h"

/* ------------ manual configuration settings -------------------------- */

/* Define TT_CONFIG_OPTION_EXTENSIONS if you want to generate code to   */
/* support separately-linkable extensions to the core engine.           */
/* Extensions are located in the freetype/lib/extend directory and      */
/* must be linked with and inited by your application                   */
#define TT_CONFIG_OPTION_EXTENSIONS

/* Define TT_CONFIG_OPTION_VERTICAL_LAYOUT if you want to generate code */
/* to support vertical layouts and data within the engine.              */
#define  TT_CONFIG_OPTION_VERTICAL_LAYOUT

/* Define TT_CONFIG_OPTION_GRAY_SCALING if you want to generate the code */
/* that supports gray-scaling and pixmap generation. Undefining this     */
/* flag will remove the following functions from the API :               */
/*   TT_Set_Raster_Palette, TT_Get_Glyph_Pixmap, TT_Get_Outline_Pixmap   */
/*                                                                       */
/* as well as some other code within the engine                          */
/*                                                                       */
#define  TT_CONFIG_OPTION_GRAY_SCALING

/* Define TT_CONFIG_OPTION_THREAD_SAFE if you want to build a thread     */
/* safe version of the engine. You'll need to specialize the ttmutex     */
/* component for your platform.                                          */
/*                                                                       */
/* RELEASE NOTE : The engine doesn't support this flag yet !!            */
#undef TT_CONFIG_OPTION_THREAD_SAFE

/* Define TT_CONFIG_OPTION_REENTRANT if you want to build a reentrant    */
/* version of the engine. You'll need to specialize the ttmutex          */
/* component for your platform.                                          */
/*                                                                       */
/* RELEASE NOTE : The engine doesn't support this flag yet !!            */
/*                                                                       */
#undef TT_CONFIG_OPTION_REENTRANT


/* Define DEBUG if you want the program to output a series of messages   */
/* to stderr regarding its behaviour.  Only useful during development.   */

/* #define DEBUG */  


/* ------------ arithmetic and processor support - ttcalc, ttraster ---- */

/* Define _GNUC_LONG64_ if you want to enable the use of the 'long long' */
/* 64-bit type provided by gcc.  Note that:                              */
/*                                                                       */
/*   1. The type isn't ANSI, and thus will produce many warnings         */
/*      during library compilation.                                      */
/*                                                                       */
/*   2. Though the generated object files are slightly smaller, the      */
/*      resulting executables are bigger of about 4Kb! gcc must be       */
/*      linking some extra code in there!                                */
/*                                                                       */
/*   3. There is really no speed gain in doing so (but it may help       */
/*      debug the ttcalc component).                                     */
/*                                                                       */
/* IMPORTANT NOTE: You don't need to define it on 64-bits machines!      */

/* #define _GNUC_LONG64_ */


/* define ALIGNMENT to your processor/environment preferred alignment */
/* size. A value of 8 should work on all current processors, even     */
/* 64-bits ones.                                                      */

#define ALIGNMENT 8


/* ------------ rasterizer configuration ----- ttraster ----------------- */

/* Define this if you want to generate a debug version of the rasterizer. */
/* This will progressively draw the glyphs while the computations are     */
/* done directly on the graphics screen... (with inverted glyphs)         */
/*                                                                        */

/* IMPORTANT: This is reserved to developers willing to debug the */
/*            rasterizer, which seems working very well in its    */
/*            current state...                                    */

/* #define DEBUG_RASTER */



/* --------------- automatic setup -- don't touch ------------------ */

/* A Note regarding single-object-compilation :                          */
/*                                                                       */
/*   Some macros have been added to allow the compilation of the whole   */
/*   TrueType engine into a single object file. This helps produce       */
/*   smaller library and shared objects files, as it gets rids of all    */
/*   'intermediate' external functions that should never be seen by      */
/*   client applications.                                                */
/*                                                                       */
/*   The macros are :                                                    */
/*                                                                       */
/*      LOCAL_DEF  :   used in header files (.h) before internal         */
/*                     functions declarations. It reverts to the         */
/*                     'extern' keyword in normal mode, and to           */
/*                     'static' in single-object mode.                   */
/*                                                                       */
/*      LOCAL_FUNC :   used in source files (.c) before internal         */
/*                     functions implementations. It reverts to          */
/*                     nothing in normal mode, and 'static' in           */
/*                     single-object mode.                               */
/*                                                                       */
/*   Hence, a ugly hack like the one used in ftcommon.h isn't needed     */
/*   anymore, as it becomes possible to hide internal functions to       */
/*   client applications.                                                */
/*                                                                       */
/*   Look for the arch/os2/freetype.c file to see how this can be        */
/*   done.                                                               */
/*                                                                       */
#ifndef SINGLE_OBJECT_COMPILATION
#define LOCAL_FUNC   /* void */
#define LOCAL_DEF    extern
#else
#define LOCAL_FUNC   static
#define LOCAL_DEF    static
#endif

/* Some systems can't use vfprintf for error messages on stderr; if  */
/* HAVE_PRINT_FUNCTION is defined, the Print macro must be supplied  */
/* externally (having the same parameters).                          */

#ifndef HAVE_PRINT_FUNCTION
#define Print( format, ap )  vfprintf( stderr, (format), (ap) )
#endif

#define FT_BIG_ENDIAN     4321
#define FT_LITTLE_ENDIAN  1234

#ifdef WORDS_BIGENDIAN
#define FT_BYTE_ORDER  FT_BIG_ENDIAN
#else
#define FT_BYTE_ORDER  FT_LITTLE_ENDIAN
#endif


/* -------------------- table management configuration ------------ */


#if defined(TT_CONFIG_OPTION_THREAD_SAFE) || \
    defined(TT_CONFIG_OPTION_REENTRANT)
#define TT_CONFIG_OPTION_THREADS
#endif

/* Defining none of these two flags produces a single-thread version of */
/* the library.                                                         */

#endif /* TTCONFIG_H */


/* END */

