# This file is part of the FreeType project.
#
# It builds the library and test programs for emx-gcc under OS/2
#
# You will need dmake.
#
# Use this file while in the 'test' directory with the following statement:
#
#   dmake -r -f arch/os2/Makefile.dm

ARCH = arch/os2
FT_MAKEFILE = $(ARCH)/Makefile.dm
MAKE = dmake -r

.IMPORT: COMSPEC
SHELL := $(COMSPEC)
SHELLFLAGS := /c
GROUPSHELL := $(SHELL)
GROUPFLAGS := $(SHELLFLAGS)
GROUPSUFFIX := .cmd
SHELLMETAS := *"?<>&|

CC = gcc

LIBDIR = ../lib
LIBDIR_DOS = ..\lib
INCDIRS = -I$(LIBDIR) -I$(LIBDIR)/$(ARCH) -I. -I$(LIBDIR)/extend

# CFLAGS = -Wall -ansi -O2 -g $(INCDIRS)
CFLAGS = -ansi -Wall -O2 -s -Zcrtdll $(INCDIRS)

# full-screen MSDOS driver
GFSDRIVER_SRC = $(ARCH)/gfs_os2.c
GFSDRIVER = $(ARCH)/gfs_os2.o
GPMDRIVER_SRC = $(ARCH)/gpm_os2.c
GPMDRIVER = $(ARCH)/gpm_os2.o
GPM_DEF = gpm_os2.def

DISPLAYSRC = display.c

SRC = gmain.c display.c \
      ftlint.c fttimer.c ftview.c ftzoom.c ftdump.c ftstring.c ftstrpnm.c \
      $(GFSDRIVER_SRC) $(GPMDRIVER_SRC)

GFSOBJS = gmain.o display.o $(GFSDRIVER)
GPMOBJS = gmain.o display.o $(GPMDRIVER)

%.o: %.c
	$(CC) $(CFLAGS) -c -o $@ $<

RESPONSE = @$(mktmp $(^:t"\n")\n)

# graphics utility and test driver

all: freetype \
     ftview.exe ftviewfs.exe \
     ftlint.exe \
     fttimer.exe fttimefs.exe \
     ftdump.exe \
     ftstring.exe ftstrfs.exe \
     ftstrpnm.exe \
     ftzoom.exe ftzoomfs.exe

fs: ftviewfs.exe ftlint.exe fttimefs.exe ftstrfs.exe ftzoomfs.exe

pm: ftview.exe ftlint.exe fttimer.exe ftstring.exe ftzoom.exe

freetype:
[
	cd $(LIBDIR_DOS)
	$(MAKE) -f $(FT_MAKEFILE) all
	cd $(MAKEDIR)
]

$(GFSDRIVER): $(GFSDRIVER_SRC)
	$(CC) -c $(CFLAGS) -o $@ $^

$(GPMDRIVER): $(GPMDRIVER_SRC)
	$(CC) -c $(CFLAGS) -o $@ $^

ftzoom.exe: $(GPMOBJS) ftzoom.o $(LIBDIR)/libttf.a $(ARCH)/$(GPM_DEF)
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftzoomfs.exe: $(GFSOBJS) ftzoom.o $(LIBDIR)/libttf.a
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftview.exe: $(GPMOBJS) ftview.o common.o $(LIBDIR)/libttf.a $(ARCH)/$(GPM_DEF)
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftviewfs.exe: $(GFSOBJS) ftview.o common.o $(LIBDIR)/libttf.a
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftlint.exe: ftlint.o $(LIBDIR)/libttf.a
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftdump.exe: ftdump.o $(LIBDIR)/libttf.a
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftstring.exe: $(GPMOBJS) ftstring.o common.o $(LIBDIR)/libttf.a \
              $(ARCH)/$(GPM_DEF)
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftstrfs.exe: $(GFSOBJS) ftstring.o common.o $(LIBDIR)/libttf.a
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

ftstrpnm.exe: ftstrpnm.o $(LIBDIR)/libttf.a
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

fttimer.exe: $(GPMOBJS) fttimer.o $(LIBDIR)/libttf.a $(ARCH)/$(GPM_DEF)
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)

fttimefs.exe: $(GFSOBJS) fttimer.o $(LIBDIR)/libttf.a
	$(CC) $(CFLAGS) -o $@ $(RESPONSE)


clean: do_clean
[
	cd $(LIBDIR_DOS)
	$(MAKE) -f $(FT_MAKEFILE) clean
	cd $(MAKEDIR)
]

distclean: do_clean do_distclean
[
	cd $(LIBDIR_DOS)
	$(MAKE) -f $(FT_MAKEFILE) distclean
	cd $(MAKEDIR)
]

do_distclean:
-[
	del dep.end
	del *.exe
	del core
]

do_clean:
-[
	del *.o
	del arch\os2\*.o
]

# depend:
#	$(MAKE) -C $(LIBDIR) -f $(LIBDIR)/$(FT_MAKEFILE) depend
#	$(CC) -E -M $(INCDIRS) $(SRC) > dep.end
#
# ifeq (dep.end,$(wildcard dep.end))
#   include dep.end
# endif

# end of Makefile.dm
